{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# V2\n",
    "\n",
    "## Instrucciones\n",
    "\n",
    "El parcial consta de 3 puntos donde deben resolver solo 2 de los 3, el tercero es obligatorio y eligen entre el primer y segundo punto, en ambos deben crear el diagrama UML. Responde a cada uno de manera detallada, implementando la solución solicitada en el lenguaje de programación Java. Asegúrate de utilizar los conceptos solicitados en cada pregunta. En el punto 3, incluye un diagrama UML de la solución que propongas.\n",
    "\n",
    "\n",
    "## Punto 1: Herencia, Clases Abstractas e Interfaces\n",
    "\n",
    "**Objetivo:**\n",
    "Implementar un sistema que simule una biblioteca con diferentes tipos de recursos (libros, revistas, y videos) utilizando herencia, clases abstractas e interfaces.\n",
    "\n",
    "**Requerimientos:**\n",
    "- Crea una clase abstracta `Recurso` con atributos `titulo` y `añoPublicacion`, y métodos abstractos `prestar()` y `devolver()`.\n",
    "- Implementa tres subclases: `Libro`, `Revista` y `Video`, cada una con atributos y comportamiento específicos (ej: `Libro` tiene `autor`, `Revista` tiene `editor`, `Video` tiene `duracion`).\n",
    "- Crea una interfaz `Digitalizable` con un método `descargar()`. Implementa esta interfaz en la clase `Video`.\n",
    "- En el método `main`, crea un arreglo de diferentes recursos y recorre el arreglo usando polimorfismo para prestar y devolver cada recurso.\n",
    "\n",
    "**Indicaciones adicionales:**\n",
    "- En tu implementación, debes manejar correctamente el uso de clases abstractas e interfaces.\n",
    "- Explica cómo el uso de la interfaz `Digitalizable` permite a `Video` tener comportamientos adicionales que no tienen las otras subclases.\n",
    "\n",
    "**Diagrama UML:**\n",
    "Dibuja el diagrama UML de clases que muestre la relación entre `Recurso`, `Libro`, `Revista`, y `Video`. Asegúrate de incluir las clases, atributos, métodos, y las relaciones de herencia y uso de la interfaz.\n",
    "\n",
    "### Solución\n",
    "\n",
    "#### Clases"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {
    "tags": [
     "hide-cell"
    ]
   },
   "outputs": [],
   "source": [
    "interface Digitalizable{\n",
    "  public void descargar();\n",
    "}\n",
    "\n",
    "abstract class Recurso{\n",
    "  String titulo;\n",
    "  int añoPublicacion;\n",
    "  String tipo;\n",
    "\n",
    "  public Recurso(String titulo, int añoPublicacion, String tipo){\n",
    "    this.titulo = titulo;\n",
    "    this.añoPublicacion = añoPublicacion;\n",
    "    this.tipo = tipo;\n",
    "  }\n",
    "\n",
    "  abstract public void prestar();\n",
    "  abstract public void devolver(); \n",
    "}\n",
    "\n",
    "class Libro extends Recurso{\n",
    "  public String autor;\n",
    "  \n",
    "  public Libro(String titulo, int añoPublicacion, String autor){\n",
    "    super(titulo, añoPublicacion, \"Libro\");\n",
    "    this.autor = autor;\n",
    "  }\n",
    "\n",
    "  @Override\n",
    "  public void prestar(){\n",
    "    System.out.print(\"Prestando %s %s.\".formatted(tipo, titulo));\n",
    "  };\n",
    "  \n",
    "  @Override\n",
    "  public void devolver(){\n",
    "    System.out.print(\"Devolviendo %s %s.\".formatted(tipo, titulo));\n",
    "  };\n",
    "}\n",
    "\n",
    "class Revista extends Recurso{\n",
    "  public String editorial;\n",
    "  \n",
    "  public Revista(String titulo, int añoPublicacion, String editorial){\n",
    "    super(titulo, añoPublicacion, \"Revista\");\n",
    "    this.editorial = editorial;\n",
    "  }\n",
    "\n",
    "  @Override\n",
    "  public void prestar(){\n",
    "    System.out.print(\"Prestando %s %s de la editorial %s.\".formatted(tipo, titulo, editorial));\n",
    "  };\n",
    "  \n",
    "  @Override\n",
    "  public void devolver(){\n",
    "    System.out.print(\"Devolviendo %s %s de la editorial %s.\".formatted(tipo, titulo, editorial));\n",
    "  };\n",
    "}\n",
    "\n",
    "class Video extends Recurso implements Digitalizable{\n",
    "  public double duracion;\n",
    "  \n",
    "  public Video(String titulo, int añoPublicacion, double duracion){\n",
    "    super(titulo, añoPublicacion, \"Video\");\n",
    "    this.duracion = duracion;\n",
    "  }\n",
    "\n",
    "  @Override\n",
    "  public void prestar(){\n",
    "    System.out.print(\"Prestando %s %s con duración %.1f.\".formatted(tipo, titulo, duracion));\n",
    "  };\n",
    "  \n",
    "  @Override\n",
    "  public void devolver(){\n",
    "    System.out.print(\"Devolviendo %s %s con duración %.1f.\".formatted(tipo, titulo, duracion));\n",
    "  };\n",
    "\n",
    "  @Override\n",
    "  public void descargar(){\n",
    "    System.out.print(\"Descargando \".formatted(tipo));\n",
    "  }\n",
    "}"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Clase de Prueba"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {
    "tags": [
     "hide-cell"
    ]
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Prestando Libro Harry Potter and the Philosopher's Stone.\n",
      "Devolviendo Libro Harry Potter and the Philosopher's Stone.\n",
      "\n",
      "Prestando Revista SOHO version 1 de la editorial SOHO.\n",
      "Devolviendo Revista SOHO version 1 de la editorial SOHO.\n",
      "\n",
      "Prestando Video Harry Potter and the Sorcerer's Stone. con duración 152,0.\n",
      "Devolviendo Video Harry Potter and the Sorcerer's Stone. con duración 152,0.\n",
      "Descargando Video\n"
     ]
    }
   ],
   "source": [
    "public class TestPunto1{\n",
    "  public static void main(String[] args){\n",
    "    Libro HarryPotter = new Libro(\"Harry Potter and the Philosopher's Stone\", 1997, \"J. K. Rowling\");\n",
    "    HarryPotter.prestar();\n",
    "    HarryPotter.devolver();\n",
    "    // System.out.println(\"\");\n",
    "\n",
    "    Revista SOHO = new Revista(\"SOHO version 1\", 2024, \"SOHO\");\n",
    "    SOHO.prestar();\n",
    "    SOHO.devolver();\n",
    "    // System.out.println(\"\");\n",
    "\n",
    "    Video HarryPotterMovie = new Video(\"Harry Potter and the Sorcerer's Stone.\", 2001, 152);\n",
    "    HarryPotterMovie.prestar();\n",
    "    HarryPotterMovie.devolver();\n",
    "    HarryPotterMovie.descargar();\n",
    "  } \n",
    "}\n",
    "\n",
    "new TestPunto1.main(null);"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Diagrama UML\n",
    "\n",
    "```{toggle} Diagrama UML de clases\n",
    "<figure>\n",
    "  <img src=\"../../../images/exam1_Punto1V2.png\" alt=\"\" style=\"width:80%\">\n",
    "  <figcaption> </figcaption>\n",
    "</figure> \n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Punto 2: Diagramas UML y Aplicación Completa\n",
    "\n",
    "**Objetivo:**\n",
    "\n",
    "Desarrollar una solución completa para un sistema que gestione diferentes tipos de electrodomésticos utilizando herencia e interfaces. Luego, realizar el diagrama UML de clases correspondiente.\n",
    "\n",
    "**Requerimientos:**\n",
    "- Crea una clase `Electrodomestico` con los atributos `marca`, `modelo` y `consumoEnergetico`.\n",
    "- Crea dos subclases: `Lavadora` y `Refrigerador`. `Lavadora` tiene un atributo adicional `capacidadCarga`, y `Refrigerador` tiene el atributo `volumen`.\n",
    "- Crea una interfaz `Operable` que tenga el método `encender()`. Tanto `Lavadora` como `Refrigerador` deben implementar esta interfaz.\n",
    "- En el método `main`, crea instancias de `Lavadora` y `Refrigerador`, y utiliza polimorfismo para invocar el método `encender()`.\n",
    "\n",
    "**Diagrama UML:**\n",
    "\n",
    "Dibuja el diagrama UML de clases que muestre la relación entre `Electrodomestico`, `Lavadora`, `Refrigerador` y la interfaz `Operable`. Asegúrate de incluir las clases, atributos, métodos, y las relaciones de herencia e implementación de la interfaz.\n",
    "\n",
    "### Solución\n",
    "\n",
    "#### Clases e Interfaces"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {
    "tags": [
     "hide-cell"
    ]
   },
   "outputs": [],
   "source": [
    "interface Operable{\n",
    "  public void encender();\n",
    "}\n",
    "\n",
    "class Electrodomestico implements Operable{\n",
    "  String marca;\n",
    "  String modelo;\n",
    "  double consumoEnergetico;\n",
    "\n",
    "  public Electrodomestico(){};\n",
    "  public Electrodomestico(String marca, String modelo, double consumoEnergetico){\n",
    "    this.marca = marca;\n",
    "    this.modelo = modelo;\n",
    "    this.consumoEnergetico = consumoEnergetico;\n",
    "  }\n",
    "\n",
    "  public void encender(){\n",
    "    System.out.print(\"El electrodomestico de marca %s, modelo %s, esta ENCENDIDO y tiene un consumo energético de %.0f W.\".formatted(marca, modelo, consumoEnergetico));\n",
    "  };\n",
    "\n",
    "  public static void main(String[] args){\n",
    "    Electrodomestico microondas = new Electrodomestico(\"Electrolux\", \"EMDO17S3GSLUW\", 700);\n",
    "    microondas.encender();\n",
    "  \n",
    "    Refrigerador nevera = new Refrigerador(\"LG\", \"334 LT Door\", 27.9, 334);\n",
    "    nevera.encender();\n",
    "  \n",
    "    Lavadora lavadora = new Lavadora(\"Samsung\", \"WF22R6270AW/CO\", 1.5, 22);\n",
    "    lavadora.encender();\n",
    "  }\n",
    "}\n",
    "\n",
    "class Lavadora extends Electrodomestico{\n",
    "  int capacidadCarga;\n",
    "\n",
    "  public Lavadora(String marca, String modelo, double consumoEnergetico, int capacidadCarga){\n",
    "    super(marca, modelo, consumoEnergetico);\n",
    "    this.capacidadCarga = capacidadCarga;\n",
    "  }\n",
    "\n",
    "  public void encender(){\n",
    "    System.out.print(\"La lavadora de marca %s, modelo %s, esta ENCENDIDO. Tiene un consumo energético de %.0f KWh/mes y una capacidad de caga de %d Kg.\".formatted(marca, modelo, consumoEnergetico, capacidadCarga));\n",
    "  };\n",
    "}\n",
    "\n",
    "class Refrigerador extends Electrodomestico{\n",
    "  int volumen;\n",
    "\n",
    "  public Refrigerador(String marca, String modelo, double consumoEnergetico, int volumen){\n",
    "    super(marca, modelo, consumoEnergetico);\n",
    "    this.volumen = volumen;\n",
    "  }\n",
    "\n",
    "  public void encender(){\n",
    "    System.out.print(\"El refrigerador de marca %s, modelo %s, esta ENCENDIDO. Tiene un consumo energético de %.0f y un volumen de %d m^3.\".formatted(marca, modelo, consumoEnergetico, volumen));\n",
    "  };\n",
    "}"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Prueba"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {
    "tags": [
     "hide-cell"
    ]
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "El electrodomestico de marca Electrolux, modelo EMDO17S3GSLUW, esta ENCENDIDO y tiene un consumo energético de 700 W.\n",
      "El refrigerador de marca LG, modelo 334 LT Door, esta ENCENDIDO. Tiene un consumo energético de 28 y un volumen de 334 m^3.\n",
      "La lavadora de marca Samsung, modelo WF22R6270AW/CO, esta ENCENDIDO. Tiene un consumo energético de 2 KWh/mes y una capacidad de caga de 22 Kg.\n"
     ]
    }
   ],
   "source": [
    "Electrodomestico pruebaElectrodomestico = new Electrodomestico();\n",
    "pruebaElectrodomestico.main(null);"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Diagrama UML\n",
    "\n",
    "```{toggle} Diagrama UML de clases\n",
    "<figure>\n",
    "  <img src=\"../../../images/exam1_Punto2V2.png\" alt=\"\" style=\"width:80%\">\n",
    "  <figcaption> </figcaption>\n",
    "</figure> \n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Punto 3: De Diagrama UML a Código\n",
    "\n",
    "Implementen el código del siguiente diagrama UML de clases\n",
    "\n",
    "<figure>\n",
    "  <img src=\"https://teach-ict.com/as_as_computing/ocr/H447/F453/3_3_6/defining_syntax/miniweb/images/UMLinheritdiagram.gif\" alt=\"\" style=\"width:90%\">\n",
    "  <figcaption> </figcaption>\n",
    "</figure> \n",
    "\n",
    "### Solución\n",
    "\n",
    "#### Clases"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {
    "tags": [
     "hide-cell"
    ]
   },
   "outputs": [],
   "source": [
    "import java.util.Date;\n",
    "\n",
    "class Employee{\n",
    "  private String name;\n",
    "  private Date started;\n",
    "\n",
    "  public Employee(String name, String started){\n",
    "    this.name = name;\n",
    "    this.started = new Date(started);\n",
    "  }\n",
    "\n",
    "  public void setName(String name){\n",
    "    this.name = name;\n",
    "  }\n",
    "\n",
    "  public void setStarted(String started){\n",
    "    this.started = new Date(started);\n",
    "  }\n",
    "}\n",
    "\n",
    "class FullTimeEmployee extends Employee{\n",
    "  private int pensiontier;\n",
    "\n",
    "  public FullTimeEmployee(String name, String started, int pensiontier){\n",
    "    super(name, started);\n",
    "    this.pensiontier = pensiontier;\n",
    "  };\n",
    "\n",
    "  public void setPensionTier(int pensiontier){\n",
    "    this.pensiontier = pensiontier;\n",
    "  }\n",
    "}\n",
    "\n",
    "class PartTimeEmployee extends Employee{\n",
    "  private int hoursworked;\n",
    "\n",
    "  public PartTimeEmployee(String name, String started, int hoursworked){\n",
    "    super(name, started);\n",
    "    this.hoursworked = hoursworked;\n",
    "  };\n",
    "\n",
    "  public void setHoursWorked(int hoursworked){\n",
    "    this.hoursworked = hoursworked;\n",
    "  }\n",
    "}"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Clases de Prueba"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {
    "tags": [
     "hide-cell"
    ]
   },
   "outputs": [],
   "source": [
    "// probando clases\n",
    "Employee sebas = new Employee(\"Sebastian\", \"05/05/2024\");\n",
    "FullTimeEmployee sebas = new FullTimeEmployee(\"Sebastian\", \"05/05/2024\", 1);\n",
    "PartTimeEmployee sebas = new PartTimeEmployee(\"Sebastian\", \"05/05/2024\", 20);"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Java",
   "language": "java",
   "name": "java"
  },
  "language_info": {
   "codemirror_mode": "java",
   "file_extension": ".jshell",
   "mimetype": "text/x-java-source",
   "name": "java",
   "pygments_lexer": "java",
   "version": "21.0.4+7-Ubuntu-1ubuntu224.04"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
